/*****
This program capture audio signal from a device and store in WAV format

WAV file structure
Wave files use the standard RIFF structure which groups the files contents (sample format, digital audio samples, etc.) 
into separate chunks, each containing it's own header and data bytes. The chunk header specifies the type and size of the chunk data bytes. 
This organization method allows programs that do not use or recognize particular types of chunks to easily skip over them and 
continue processing following known chunks. Certain types of chunks may contain sub-chunks. For example, in the diagram to the right, 
you can see that the "fmt " and "data" chunks are actually sub-chunks of the "RIFF" chunk. 
One tricky thing about RIFF file chunks is that they must be word aligned. This means that their total size must be a multiple of 2 bytes 
(ie. 2, 4, 6, 8, and so on). If a chunk contains an odd number of data bytes, causing it not to be word aligned, 
an extra padding byte with a value of zero must follow the last data byte. This extra padding byte is not counted in the chunk size, 
therefor a program must always word align a chunk headers size value in order to calculate the offset of the following chunk. 
****/

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <iostream>
#include "al.h"
#include "alc.h"

using namespace std;

#define	OUTPUT_WAVE_FILE		"Capture.wav"
#define BUFFERSIZE				4410

///*  Standard multimedia window api from "mmsystem.h"
// *  extended waveform format structure used for all non-PCM formats. this
// *  structure is common to all non-PCM formats.
// */
//typedef struct tWAVEFORMATEX
//{
//    WORD        wFormatTag;         /* format type */
//    WORD        nChannels;          /* number of channels (i.e. mono, stereo...) */
//    DWORD       nSamplesPerSec;     /* sample rate */
//    DWORD       nAvgBytesPerSec;    /* for buffer estimation */
//    WORD        nBlockAlign;        /* block size of data */
//    WORD        wBitsPerSample;     /* number of bits per sample of mono data */
//    WORD        cbSize;             /* the count in bytes of the size of */
//                                    /* extra information (after cbSize) */
//} WAVEFORMATEX;

typedef struct
{
	char			RIFFId[4];
	long			RIFFSize;
	char			WaveId[4];
	char			FmtId[4];
	long			FmtSize;
	WAVEFORMATEX	wfex;
	char			DataId[4];
	long			DataSize;
} WAVEHEADER;

int main()
{
	ALCdevice		*device;
	ALCcontext		*context;
	ALCdevice		*capDevice;
	const ALCchar	*capDeviceName;
	WAVEHEADER		wavHeader;
	ALchar			Buffer[BUFFERSIZE];
	FILE			*outFile;
	
	ALint numSamples, tmpSize, dataSize = 0;

	// Get list of available Capture Devices
	const ALchar *deviceList = alcGetString(NULL, ALC_CAPTURE_DEVICE_SPECIFIER);
	if (deviceList)
	{
		cout << "\nAvailable Capture Devices are:-\n ";
		while (*deviceList)
		{
			cout << deviceList << endl;
			deviceList += strlen(deviceList) + 1;
		}
	}

	// Get the name of the 'default' capture device
	capDeviceName = alcGetString(NULL, ALC_CAPTURE_DEFAULT_DEVICE_SPECIFIER);
	cout << "\nDefault Capture Device is \n" << capDeviceName << endl;

	// Open the default Capture device to record a 22050Hz 16bit Mono Stream using an internal buffer
	// of BUFFERSIZE Samples (== BUFFERSIZE * 2 bytes)
	device = alcCaptureOpenDevice(capDeviceName, 22050, AL_FORMAT_MONO16, BUFFERSIZE);
	if (device)
	{
		// Create / open a file for the captured data
		outFile = fopen(OUTPUT_WAVE_FILE, "wb");

		// Prepare a WAVE file header for the captured data
		sprintf(wavHeader.RIFFId, "RIFF");
		wavHeader.RIFFSize = 0;
		sprintf(wavHeader.WaveId, "WAVE");
		sprintf(wavHeader.FmtId, "fmt ");
		wavHeader.FmtSize = sizeof(WAVEFORMATEX);		
		wavHeader.wfex.nChannels = 1;
		wavHeader.wfex.wBitsPerSample = 16;
		wavHeader.wfex.wFormatTag = WAVE_FORMAT_PCM;
		wavHeader.wfex.nSamplesPerSec = 22050;
		wavHeader.wfex.nBlockAlign = wavHeader.wfex.nChannels * wavHeader.wfex.wBitsPerSample / 8;
		wavHeader.wfex.nAvgBytesPerSec = wavHeader.wfex.nSamplesPerSec * wavHeader.wfex.nBlockAlign;
		wavHeader.wfex.cbSize = 0;
		sprintf(wavHeader.DataId, "data");
		wavHeader.DataSize = 0;

		fwrite(&wavHeader, sizeof(WAVEHEADER), 1, outFile);

		// Start audio capture
		alcCaptureStart(device);
		cout << "start capturing ... " << endl;

		// Record until a key is pressed
		while(!_kbhit())
		{
			// Release some CPU time ...
			Sleep(1);

			// Find out how many samples have been captured
			alcGetIntegerv(device, ALC_CAPTURE_SAMPLES, 1, &numSamples);

			//ALFWprintf("Samples available : %d\r", iSamplesAvailable);

			// When we have enough data to fill our BUFFERSIZE byte buffer, grab the samples
			if (numSamples > (BUFFERSIZE / wavHeader.wfex.nBlockAlign))
			{
				// Consume Samples
				alcCaptureSamples(device, Buffer, BUFFERSIZE / wavHeader.wfex.nBlockAlign);

				// Write the audio data to a file
				fwrite(Buffer, BUFFERSIZE, 1, outFile);

				// Record total amount of data recorded
				dataSize += BUFFERSIZE;
			}
		}

		// Stop capture
		alcCaptureStop(device);
		cout << "Stop capturing ... " << endl;

		// Fill in Size information in Wave Header
		fseek(outFile, 4, SEEK_SET);
		tmpSize = dataSize + sizeof(WAVEHEADER) - 8;
		fwrite(&tmpSize, 4, 1, outFile);
		fseek(outFile, 42, SEEK_SET);
		fwrite(&dataSize, 4, 1, outFile);

		fclose(outFile);

		//ALFWprintf("\nSaved captured audio data to '%s'\n", OUTPUT_WAVE_FILE);
		cout << "audio file saved" << endl;

		// Close the Capture Device
		alcCaptureCloseDevice(device);
	}

	return 0;
}